"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

var _webinyEntity = require("webiny-entity");

var _entities = require("webiny-api/entities");

var _graphql = require("webiny-api/graphql");

var _webinyApiSecurity = require("webiny-api-security");

var _got = _interopRequireDefault(require("got"));

const Mailchimp = function ({
  apiKey
}) {
  this.apiKey = apiKey;

  this.isValidApiKey = async () => {
    try {
      await this.get({
        path: `/lists/`
      });
      return true;
    } catch (e) {
      return false;
    }
  };

  this.get = ({
    path
  }) => {
    return this.request({
      path,
      method: "get"
    });
  };

  this.post = ({
    path,
    body
  }) => {
    return this.request({
      path,
      body,
      method: "post"
    });
  };

  this.request = ({
    path,
    method,
    body
  }) => {
    // eslint-disable-next-line
    const [hash, dataCenter] = this.apiKey.split("-");
    return (0, _got.default)(`https://${dataCenter}.api.mailchimp.com/3.0` + path, {
      method,
      json: true,
      body,
      headers: {
        authorization: "apikey " + this.apiKey
      }
    });
  };
};

const mailchimpSettingsModelFactory = parent => {
  var _temp;

  return _temp = class MailchimpSettingsModel extends _webinyEntity.EntityModel {
    constructor() {
      super();
      (0, _defineProperty2.default)(this, "apiKey", void 0);
      (0, _defineProperty2.default)(this, "enabled", void 0);
      this.setParentEntity(parent);
      this.attr("enabled").boolean();
      this.attr("apiKey").char().onSet(value => {
        if (value && value !== this.apiKey) {
          this.getParentEntity().on("beforeSave", async () => {
            const mailchimp = new Mailchimp({
              apiKey: value
            });
            const valid = await mailchimp.isValidApiKey();

            if (!valid) {
              throw Error("API key invalid.");
            }
          }).setOnce();
        }

        return value;
      });
    }

  }, _temp;
};

var _default = [{
  name: "schema-settings-mailchimp",
  type: "schema-settings",
  namespace: "mailchimp",
  typeDefs:
  /* GraphQL */
  `
            type MailchimpSettings {
                enabled: Boolean
                apiKey: String
            }

            type MailchimpSettingsResponse {
                data: MailchimpSettings
                error: Error
            }

            input MailchimpSettingsInput {
                enabled: Boolean
                apiKey: String
            }

            extend type SettingsQuery {
                mailchimp: MailchimpSettingsResponse
            }

            extend type SettingsMutation {
                mailchimp(data: MailchimpSettingsInput): MailchimpSettingsResponse
            }
        `,
  entity: ({
    mailchimp: {
      entities: {
        MailchimpSettings
      }
    }
  }) => MailchimpSettings
}, {
  type: "entity",
  name: "entity-mailchimp-settings",
  namespace: "mailchimp",
  entity: {
    name: "MailchimpSettings",
    factory: (...args) => {
      var _class2, _temp2;

      return _temp2 = _class2 = class MailchimpSettings extends (0, _entities.settingsFactory)(...args) {
        constructor() {
          super();
          (0, _defineProperty2.default)(this, "data", void 0);
          (0, _defineProperty2.default)(this, "load", void 0);
          this.attr("data").model(mailchimpSettingsModelFactory(this));
        }

      }, (0, _defineProperty2.default)(_class2, "key", "mailchimp"), _temp2;
    }
  }
}, {
  type: "graphql",
  name: "graphql-mailchimp",
  namespace: "mailchimp",
  security: {
    shield: {
      SettingsQuery: {
        mailchimp: (0, _webinyApiSecurity.hasScope)("cms:settings")
      },
      SettingsMutation: {
        mailchimp: (0, _webinyApiSecurity.hasScope)("cms:settings")
      },
      MailchimpQuery: {
        listLists: (0, _webinyApiSecurity.hasScope)("cms:editor")
      }
    }
  },
  typeDefs: () => [
  /* GraphQL */
  `
                type AddToListResponse {
                    error: Error
                }

                type MailchimpMutation {
                    # Add a new member to members list.
                    addToList(list: String!, email: String!): AddToListResponse
                }

                type Mutation {
                    mailchimp: MailchimpMutation
                }

                type List {
                    id: String
                    name: String
                }

                type ListsResponse {
                    data: [List]
                    meta: ListMeta
                    error: Error
                }

                type MailchimpQuery {
                    listLists: ListsResponse
                }

                type Query {
                    mailchimp: MailchimpQuery
                }
            `],
  resolvers: () => [{
    Query: {
      mailchimp: _graphql.dummyResolver
    },
    Mutation: {
      mailchimp: _graphql.dummyResolver
    },
    MailchimpQuery: {
      listLists: async (_, args, {
        mailchimp: {
          entities
        }
      }) => {
        const {
          MailchimpSettings
        } = entities;
        const settings = await MailchimpSettings.load();

        if (!settings || !(await settings.get("data.apiKey"))) {
          throw Error("Mailchimp API key not set.");
        }

        const mailchimp = new Mailchimp({
          apiKey: settings.data.apiKey
        });

        try {
          const listsResponse = await mailchimp.get({
            path: `/lists/`
          });
          const output = listsResponse.body.lists.map(item => ({
            id: item.id,
            name: item.name
          }));
          return new _graphql.ListResponse(output);
        } catch (e) {
          return new _graphql.ListErrorResponse(e);
        }
      }
    },
    MailchimpMutation: {
      addToList: async (_, {
        list: listId,
        email
      }, {
        mailchimp: {
          entities
        }
      }) => {
        const {
          MailchimpSettings
        } = entities;
        const settings = await MailchimpSettings.load();

        if (!settings || !(await settings.get("data.apiKey"))) {
          throw Error("Mailchimp API key not set.");
        }

        const mailchimp = new Mailchimp({
          apiKey: settings.data.apiKey
        });

        try {
          const listResponse = await mailchimp.get({
            path: `/lists/${listId}`
          });
          await mailchimp.post({
            path: `/lists/${listId}/members`,
            body: {
              email_address: email,
              status: listResponse.body.double_optin ? "pending" : "subscribed"
            }
          });
        } catch (e) {
          if (e.body) {
            return new _graphql.ErrorResponse({
              message: e.body.title,
              data: {
                code: e.body.status,
                detail: e.body.detail,
                type: e.body.type
              }
            });
          }

          return new _graphql.ErrorResponse({
            message: e.message
          });
        }
      }
    }
  }]
}];
exports.default = _default;
//# sourceMappingURL=index.js.map